define('feature/changeset/tree-and-diff-view', [
    'jquery',
    'eve',
    'underscore',
    'memoir',
    'util/navbuilder',
    'util/deprecation',
    'model/conflict',
    'model/file-change',
    'model/revision',
    'model/path',
    'widget/faux-scrollbar',
    'feature/changeset/difftree',
    'feature/file-content',
    'feature/file-content/diff-image',
    'exports'
], function(
    $,
    eve,
    _,
    memoir,
    navbuilder,
    deprecated,
    Conflict,
    FileChange,
    Revision,
    Path,
    FauxScrollbar,
    difftree,
    FileContent,
    diffImage,
    exports
) {
    var DiffTree = difftree.DiffTree,
        ROOT = "ROOT";

    var _options;

    //state
    var currentCommitRange,
        currentFilePath,
        changingState = false;

    // components/features/widgets
    var currentDiffTree,
        diffTreesByCommitRangeId = {}, //cache for diff-tree's created for different CommitRanges
        fileContent;

    // Selectors for resizing changesetPanes height and scrollbar & spinner
    var $window = $(window),
        $footer,
        $content,
        $container,
        $fauxScrollbar,
        $spinner,
        windowHeight,
        diffTreeMaxHeight,
        $changesetFiles,
        $changesetFileContent,
        $fileTreeContainer,
        $fileTreeWrapper,
        $fileTree,
        $contentView,
        $diffViewToolbar,
        diffViewToolbarHeight,
        isFixed = false, // boolean for determining if the file tree is stalking or not
        offset,
        offsetTop;

    var fauxScrollbar, setFauxScrollbarPosition;

    function getFauxScrollbarPositionObject() {
        var left = $contentView.offset().left;
        return {
            bottom: 0,
            right: $window.width() - (left + $contentView.outerWidth()),
            left: left
        };
    }

    function initFauxScrollbar() {
        if (!fauxScrollbar) {
            var target = $contentView[0];
            if (target) {
                fauxScrollbar = new FauxScrollbar();
                fauxScrollbar.init($contentView[0], getFauxScrollbarPositionObject());
                setFauxScrollbarPosition = function () {
                    fauxScrollbar.setPosition(getFauxScrollbarPositionObject());
                };
                eve.on('stash.feature.changeset.difftree.collapseAnimationFinished', setFauxScrollbarPosition);
            }
        } else {
            setFauxScrollbarPosition();
        }
    }

    function destroyFauxScrollbar() {
        if (setFauxScrollbarPosition) {
            eve.off('stash.feature.changeset.difftree.collapseAnimationFinished', setFauxScrollbarPosition);
            setFauxScrollbarPosition = null;
        }
        if (fauxScrollbar) {
            fauxScrollbar.destroy();
            fauxScrollbar = null;
        }
    }

    function initFileContent($node) {
        var path = getPathFromNode($node),
            srcPath = getSrcPathFromNode($node),
            changeType = getChangeTypeFromNode($node),
            conflict = getConflictFromNode($node),
            executable = getExecutableFromNode($node),
            srcExecutable = getSrcExecutableFromNode($node);

        destroyFauxScrollbar();

        if (!fileContent) {
            fileContent = new FileContent($container, "changeset-file-content");
        }

        var fileChange = new FileChange({
            commitRange : currentCommitRange,
            srcPath : srcPath,
            path : path,
            type : changeType,
            conflict : conflict,
            srcExecutable: srcExecutable,
            executable: executable
        });

        currentFilePath = path;

        $container.height($container.height());
        //temporarily set the height explicitly to the current height to stop the jump when the diffview is removed.
        //cleaned up in onTreeAndDiffViewSizeChanged
        var scrollTop = $window.scrollTop();

        return fileContent.init(fileChange, null, null, _options).done(function() {
            $changesetFileContent = $('#changeset-file-content');
            $diffViewToolbar = $changesetFileContent.find('.source-commands');
            $contentView = $changesetFileContent.find('.content-view');
            onTreeAndDiffViewSizeChanged();

            scrollTop = scrollContentToTop(scrollTop);
            $window.scrollTop(scrollTop);
            onScrollPositionChanged(scrollTop);
        });
    }

    function destroyFileContent() {
        var deferred = $.Deferred();
        currentFilePath = null;

        destroyFauxScrollbar();

        if (fileContent) {
            fileContent.destroy();
            fileContent = null;
        }

        $("#changeset-file-content").remove();

        return deferred.resolve();
    }

    function getPathFromNode($node) {
        return new Path($node.data('path'));
    }

    function getChangeTypeFromNode($node) {
        return $node.data('changeType');
    }

    function getSrcPathFromNode($node) {
        return new Path($node.data('srcPath'));
    }

    function getConflictFromNode($node) {
        return $node.data('conflict') && new Conflict($node.data('conflict'));
    }

    function getSrcExecutableFromNode($node) {
        return $node.data('srcExecutable');
    }

    function getExecutableFromNode($node) {
        return $node.data('executable');
    }

    function onTreeAndDiffViewSizeChanged() {
        windowHeight = $window.height();
        $container.height(""); //remove any explicit height on diff+tree container to allow size to be set by diffview.
        diffViewToolbarHeight = ($diffViewToolbar && $diffViewToolbar.outerHeight()) || 0;

        if ($changesetFileContent.length) {
            offset = $changesetFileContent.offset();
            if (offset) {
                $fauxScrollbar = $('.faux-scrollbar');
                offsetTop = offset.top;

                var footerHeight = $footer.outerHeight() + parseInt($footer.css("margin-top"), 10),
                    fauxScrollbarHeight = $fauxScrollbar.outerHeight(),
                    // available - the window height available for the file tree and diff content once all the chrome space is removed
                    available = Math.floor(windowHeight - offsetTop - footerHeight - fauxScrollbarHeight),
                    // needed - the height needed to display the file-tree onscreen at all times. Since we fix it to
                    //          the window, we never need more than windowHeight. If both file tree and diff are shorter
                    //          than the window height, then we take the taller of the diff and the tree as our minimum
                    needed = Math.min(Math.max($fileTree.height(), $contentView.height()), windowHeight);
                $changesetFileContent.css('min-height', Math.max(available, needed));
                makeRoomForDiffToolbar();
            }
        }
        diffTreeMaxHeight = windowHeight - $('.diff-tree-toolbar').outerHeight();

        // update diff-tree height
        $fileTreeWrapper.css({'max-height': diffTreeMaxHeight + 'px', 'border-bottom-width': 0 });

        var diffViewHeight = $changesetFileContent.outerHeight();
        var diffTallerThanWindow = diffViewHeight > windowHeight;
        $changesetFiles.toggleClass('long-diff', diffTallerThanWindow);

        if (diffTallerThanWindow) {
            initFauxScrollbar();
        } else {
            destroyFauxScrollbar();
        }

        var treeHeight = $fileTreeWrapper.outerHeight();
        var treeTallerThanWindow = treeHeight > diffTreeMaxHeight;
        $changesetFiles.toggleClass('long-tree', treeTallerThanWindow);

        toggleFooterHiddenForFixedMode(diffTallerThanWindow || treeTallerThanWindow);
    }

    function onScrollPositionChanged(scrollTop) {
        var shouldBeFixed;
        var longDiff = Math.max($fileTreeWrapper.outerHeight(), $changesetFileContent.outerHeight()) > windowHeight;

        shouldBeFixed = scrollTop >= offsetTop && longDiff;

        if (shouldBeFixed != isFixed) {
            isFixed = shouldBeFixed;
            $changesetFiles.toggleClass('fixed', isFixed);
            makeRoomForDiffToolbar();
        }
    }

    /**
     * You may need to adjust diff padding when:
     * - you enter or exit fixed mode (We only need padding in fixed mode, since the toolbar is removed from normal
     *     document flow)
     * - The width of the window changes (path might wrap or no longer wrap)
     * - The file changes (length of path might cause wrapping or no longer wrap)
     *
     */
    function makeRoomForDiffToolbar() {
        $contentView.css({ 'padding-top': isFixed ? diffViewToolbarHeight + 'px' : '' });
    }

    function toggleFooterHiddenForFixedMode(isFixed) {
        $footer.toggleClass('hidden', isFixed);
        if (isFixed) {
            $('.aui-page-panel-content').css({'padding-bottom': 0, 'border-bottom': 0 });
        } else {
            $('.aui-page-panel-content').css({'padding-bottom': '', 'border-bottom': '' });
        }
    }

    function scrollContentToTop(scrollTop) {
        var diffOffset = $changesetFileContent.offset().top;
        return Math.min(scrollTop, diffOffset);
    }

    function onStateChange() {
        changingState = true;

        var selectedPath = getPathFromUrl();

        var selectedFileChanged = (Boolean(selectedPath) ^ Boolean(currentFilePath)) || (selectedPath && selectedPath.toString() !== currentFilePath.toString());

        if (selectedFileChanged && currentDiffTree) {
            currentDiffTree.selectFile(selectedPath.getComponents());
            var $node = currentDiffTree.getSelectedFile();
            if($node && $node.length > 0) {
                initFileContent($node);
            }
        }

        changingState = false;
    }

    function updateDiffTree(optSelectedPathComponents) {
        if (!$spinner) {
            $spinner = $("<div class='spinner'/>");
        }
        $spinner.appendTo("#content .file-tree-wrapper").spin("large");
        return currentDiffTree.init(optSelectedPathComponents).always(function() {
            if ($spinner) {
                $spinner.spinStop().remove();
                $spinner = null;
            }
        }).done(function() {
            $fileTree = $('.file-tree');
            diffTreeMaxHeight = windowHeight - $('.diff-tree-toolbar').outerHeight();
            $fileTreeWrapper.css('max-height', diffTreeMaxHeight);
        });
    }

    function getPathFromUrl() {
        return new Path(window.location.hash.substring(1));
    }

    var toggleDiffTree;
    function initDiffTreeToggle() {
        var $toggle = $(".collapse-file-tree");
        var $changesetFilesContainer = $(".changeset-files");
        var $diffTreeContainer = $(".file-tree-container");
        var $toolbarItemPrimary = $diffTreeContainer.find('.aui-toolbar2-primary');

        var collapsed;
        var timeout = ($.browser.msie && parseInt($.browser.version, 10) <= 9) ? 0 : 500; // 0.5s CSS transition duration
        var debouncedToggleEvent = _.debounce(function() {
            eve('stash.feature.changeset.difftree.collapseAnimationFinished', null, collapsed);
        }, timeout);

        toggleDiffTree = function() {
            if ($.browser.msie && parseInt($.browser.version, 10) <= 9 ) {
                /* IE 9 and below doesn't support CSS transitions, so the panel collapses immediately,
                   but while the heading fades out, the button wraps to the next line, then jumps up when the
                   animation completes, so we just toggle the visibility immediately */
                $toolbarItemPrimary.toggle();
            } else {
                /* all other browsers get a nice fade in/out of the header as the panel expands/collapses */
                $toolbarItemPrimary.fadeToggle();
            }

            $changesetFilesContainer.toggleClass('collapsed');

            collapsed = $changesetFilesContainer.hasClass('collapsed');
            eve('stash.feature.changeset.difftree.toggleCollapse', null, collapsed);
            debouncedToggleEvent();
        };

        $toggle.on('click', function(e) {
            e.preventDefault();
            toggleDiffTree();
        });
    }

    function initDiffTree() {
        $('.no-changes-placeholder').remove();

        var filePath = currentFilePath ? currentFilePath : getPathFromUrl();
        return updateDiffTree(filePath.getComponents()).pipe(function(diffTree) {
            var $node = diffTree.getSelectedFile();
            if ($node && $node.length) {
                return initFileContent($node);
            } else {
                return destroyFileContent().done(function() {
                    /* Append a placeholder <div> to keep the table-layout so that
                       the diff-tree does not consume the entire page width */
                    $('.changeset-files').append($("<div class='message no-changes-placeholder'></div>").text(stash_i18n('stash.web.no.changes.to.show', 'No changes to show')));
                });
            }
        });
    }

    exports.updateCommitRange = function(commitRange) {
        if (commitRange.getId() === currentCommitRange.getId()) {
            // bail out if not actually changing the diff.
            return;
        }

        currentCommitRange = commitRange;
        currentDiffTree.reset(); // unbind any event listeners

        if (Object.prototype.hasOwnProperty.call(diffTreesByCommitRangeId, currentCommitRange.getId())){
            // Use cached difftree if it exists.
            currentDiffTree = diffTreesByCommitRangeId[currentCommitRange.getId()];
        } else {
            currentDiffTree = new DiffTree(".file-tree-wrapper", currentCommitRange, {
                maxChanges : _options.maxChanges,
                hasOtherParents : _options.numberOfParents > 1
            });
            diffTreesByCommitRangeId[currentCommitRange.getId()] = currentDiffTree;
        }

        initDiffTree();
    };

    function onSelectedNodeChanged($node, initializingTree) {
        // Only set the hash if we're here from a user clicking a file name.
        // If it's a popState or a pushState or hashchange, the hash should already be set correctly.
        // If we're initializing a full tree, we want an empty hash.
        // If we're initializing a full tree BECAUSE of a changeState, the hash should still already be set correctly.
        if (!changingState && !initializingTree) {
            window.location.hash = $node ? getPathFromNode($node).toString() : "";
        }
    }

    function onRequestToggleDiffTreeHandler(keys) {
        if ($.browser.msie && parseInt($.browser.version, 10) === 9) {
            // IE9 crashes when you type 't'. "IE crash?! That's unpossible!" you cry.
            // But it's true. The browser executes the full call stack, then crashes before
            // debouncedToggleEvent is called. If you set a breakpoint in that call stack though, it
            // often does NOT crash. The crash doesn't occur without the 'collapsed' class being added.
            // But it only occurs when trigger from the keyboard shortcut (and setTimeout hackery doesn't help).
            return;
        }
        (this.execute ? this : AJS.whenIType(keys)).execute(toggleDiffTree);
    }

    function onRequestMoveToNextHandler(keys) {
        (this.execute ? this : AJS.whenIType(keys)).execute(function() {
            currentDiffTree.openNextFile();
        });
    }

    function onRequestMoveToPreviousHandler(keys) {
        (this.execute ? this : AJS.whenIType(keys)).execute(function() {
            currentDiffTree.openPrevFile();
        });
    }

    exports.init = function(commitRange, options) {
        _options = $.extend({}, exports.defaults, options);

        $footer = $("#footer");
        $content = $("#content");
        $container = $content.find(".changeset-files");
        $fileTreeContainer = $('.file-tree-container');
        $fileTreeWrapper = $fileTreeContainer.children('.file-tree-wrapper');
        windowHeight = $window.height();
        $changesetFiles = $('.changeset-files');
        $changesetFileContent = $('#changeset-file-content');

        diffImage.init();

        currentCommitRange = commitRange;
        currentDiffTree = new DiffTree(".file-tree-wrapper", currentCommitRange, {
            maxChanges : _options.maxChanges,
            hasOtherParents : _options.numberOfParents > 1
        });
        diffTreesByCommitRangeId[currentCommitRange.getId()] = currentDiffTree;
        currentFilePath = getPathFromUrl();

        // memoir fires a changestate event when the hash changes for browsers that support pushState...
        eve.on('memoir.changestate', onStateChange);

        // ...but for browsers that don't support pushState, we have to manually listen for it.
        // Also for IE10 which doesn't fire a popstate event for hash changes.
        // onStateChange handles the double firing
        $(window).hashchange(onStateChange);

        eve.on("window.resize.debounced", onTreeAndDiffViewSizeChanged);
        eve.on("window.scroll.throttled", onScrollPositionChanged);

        eve.on('stash.feature.changeset.difftree.selectedNodeChanged', onSelectedNodeChanged);

        initDiffTreeToggle();
        initDiffTree();

        eve.on('stash.keyboard.shortcuts.requestToggleDiffTreeHandler', onRequestToggleDiffTreeHandler);
        eve.on('stash.keyboard.shortcuts.requestMoveToNextHandler', onRequestMoveToNextHandler);
        eve.on('stash.keyboard.shortcuts.requestMoveToPreviousHandler', onRequestMoveToPreviousHandler);
    };

    exports.reset = function() {
        diffImage.reset();


        if (currentDiffTree) {
            currentDiffTree.reset();
        }

        currentCommitRange = undefined;
        currentDiffTree = undefined;
        diffTreesByCommitRangeId = {};
        currentFilePath = undefined;
        isFixed = false;

        // undo any styling from fixed positioning.
        $changesetFiles.removeClass('fixed');
        toggleFooterHiddenForFixedMode(false);

        // memoir fires a changestate event when the hash changes for browsers that support pushState...
        eve.off('memoir.changestate', onStateChange);

        // ...but for browsers that don't support pushState, we have to manually listen for it.
        if (!memoir.nativeSupport()) {
            $(window).unbind('hashchange', onStateChange);
        }

        eve.off("window.resize.debounced", onTreeAndDiffViewSizeChanged);
        eve.off("window.scroll.throttled", onScrollPositionChanged);

        eve.off('stash.feature.changeset.difftree.selectedNodeChanged', onSelectedNodeChanged);

        eve.off('stash.keyboard.shortcuts.requestToggleDiffTreeHandler', onRequestToggleDiffTreeHandler);
        eve.off('stash.keyboard.shortcuts.requestMoveToNextHandler', onRequestMoveToNextHandler);
        eve.off('stash.keyboard.shortcuts.requestMoveToPreviousHandler', onRequestMoveToPreviousHandler);

        return destroyFileContent();
    };

    /**
     * @deprecated since 2.0. Use init() instead.
     * @type {Function}
     */
    exports.onReady = deprecated(exports.init, 'feature/changeset/tree-and-diff-view::onReady');

    exports.defaults = {
        breadcrumbs : true,
        sourceLink : true,
        changeTypeLozenge : true,
        changeModeLozenge : true,
        contentType : FileContent.contentType.DIFF
    };

    exports.commentMode = FileContent.commentMode;
});
