define('feature/comments/diff-comment-container', [
    'eve',
    'util/performance',
    'feature/comments/comment-container',
    'feature/comments/comment-tips'
], function (eve, performance, CommentContainer, commentTips) {
    var padding;
    eve.on('memoir.changestate', function() {
        // when you pushstate between tabs, reset padding because diff-tab includes a margin for the add comment column
        padding = null;
    });

    return CommentContainer.extend({
        rootCommentListSelector : '.comment-list',
        initialize : function() {
            var self = this;

            CommentContainer.prototype.initialize.apply(this, arguments);
            this.$el.prev().addClass('commented-line');
        },
        render : function() {
            var $newEl = stash.feature.comments($.extend({
                comments : this.collection.toJSON(),
                tips : commentTips.tips
            }, this.anchor.toJSON()));

            this.$el.replaceWith($newEl);

            this.setElement($newEl[0]);
        },
        /**
         * NOTE: on why we use queueDOMRead and queueDOMWrite.
         * Each call to this method reads from the DOM, then writes to the DOM. If you do that in a loop, each
         * iteration causes a reflow. By queuing them up, we can ensure that all the reads happen first, then all
         * the writes. This means you only get one reflow for the lot of them.
         * @param maxWidth
         */
        setPanelMaxWidthPx : function(maxWidth) {
            var $el = this.$el;
            var $panel = $el.children('.panel');
            if (padding == null) { // calculate this once on the page, for now. Should be the same for all.
                performance.queueDOMRead(function() {
                    padding = $el.outerWidth(true) - $panel.width();
                    performance.queueDOMWrite(function () {
                        $panel.css('max-width', (maxWidth - padding) + 'px');
                    });
                });
            } else {
                performance.queueDOMWrite(function () {
                    $panel.css('max-width', (maxWidth - padding) + 'px');
                });
            }
        },
        closeCommentForm : function($form, options) {
            // try to destroy the box, unless the doNotDestory flag is set.
            // this flag is set when a comment is successfully submitted and the form is about to be replaced with a comment.
            if (!options || !options.doNotDestroy) {
                var $commentList = $form.parent().parent();

                // if this is a top-level form and there are no comments, remove the container.
                if ($commentList.is(this.rootCommentListSelector) && $commentList.children('.comment').length === 0) {
                    //TODO DRY this up - MM
                    this.deleteDraftComment(this.getDraftCommentFromForm($form));
                    return this.destroy();
                }
            }

            return CommentContainer.prototype.closeCommentForm.apply(this, arguments);
        },
        onCommentDeleted : function() {
            this.destroyIfEmpty();
        },
        destroyIfEmpty : function() {
            var $commentList = this.$(this.rootCommentListSelector);
            if ($commentList.children('.comment').length === 0 && !$commentList.find('textarea').val()) {
                return this.destroy();
            }
        },
        destroy : function() {
            padding = null;
            this.$el.prev().removeClass('commented-line');
            eve('stash.comment.comment-container.destroyed', this);
            this.context.destroy(this);
        }
    });
});