define('feature/file-content/source-view',
['jquery', 'underscore', 'eve', 'util/event', 'util/ajax', 'util/navbuilder', 'util/html', 'util/error', 'aui', 'widget/paged-scrollable', 'feature/file-content/binary-view', 'widget/faux-scrollbar', 'widget/loaded-range'],
function ($, _, eve, event, ajax, navBuilder, htmlUtil, errorUtil, AJS, PagedScrollable, binaryView, FauxScrollbar, LoadedRange) {

    event.listenForFontSizeChange();

    var isSyntaxHighlightingDisabled = $.browser.msie && parseInt($.browser.version, 10) < 9,
        stripsWhitespaceOutsidePreTags = $.browser.msie && parseInt($.browser.version, 10) < 9,
        fullLineEnd = stripsWhitespaceOutsidePreTags ? '<br /></pre>' : '<br />',
        truncatedLineEnd = '<span title="'+ stash_i18n('stash.web.source.line.truncated', 'This line has been truncated.') + '">&hellip;</span>' + fullLineEnd,
        lineOpts = {
            lineStart : stripsWhitespaceOutsidePreTags ? '<pre>' : '',
            lineEnd : function(line) {
                return line.truncated ? truncatedLineEnd : fullLineEnd;
            },
            emptyLine : ''
        },
        isIE9 = $.browser.msie && Math.floor(parseInt($.browser.version, 10)) === 9,
        pageSize = $.browser.msie ? 1000 : 5000,
        maxLinesDisplayed = 20000;


    function getData(path, untilRevision, params) {
        var restUrl = navBuilder.currentRepo().browse().path(path).at(untilRevision.getDisplayId()).build();

        return ajax.rest({
            url: restUrl,
            data: params,
            statusCode : {
                '404' : function(xhr, testStatus, errorThrown, data, fallbackError) {

                    var error = data && data.errors && data.errors.length && data.errors[0];

                    if (errorUtil.isErrorEntityWithinRepository(error)) {
                        return false; // don't handle this globally.
                    }
                }
            }
        });
    }

    var NO_HIGHLIGHT_LANGUAGE = "text";
    var getLanguage = (function () {

        var pattern = /#!(?:\/(?:usr\/)?(?:local\/)?bin\/([^\s]+))(?:\s+([^\s]+))?/,
            knownLanguages = {
                '1c' : { exts : [], exes : []},
                actionscript : { exts : ['as'], exes : []},
                apache : { exts : [], exes : []},
                avrasm : { exts : [], exes : []},
                axapta : { exts : [], exes : []},
                bash : { exts : ['sh'], exes : ['sh', 'bash', 'zsh']},
                cmake : { exts : ['makefile'], exes : []},
                coffeescript: { exts : ['coffee'], exes : []},
                cpp : { exts : ['cpp', 'h', 'c', 'cc', 'cxx','hpp'], exes : []},
                cs : { exts : ['cs'], exes : []},
                css : { exts : ['css'], exes : []},
                d : { exts : ['a'], exes : []},
                delphi : { exts : [], exes : []},
                diff : { exts : ['diff', 'patch'], exes : []},
                django : { exts : [], exes : []},
                dos : { exts : ['bat'], exes : []},
                erlang : { exts : [], exes : []},
                'erlang-repl' : { exts : [], exes : []},
                glsl : { exts : ['glslv', 'vert', 'frag', 'shader'], exes : []},
                go : { exts : ['go'], exes : []},
                haskell : { exts : ['hs', 'lhs'], exes : []},
                ini : { exts : ['ini'], exes : []},
                java : { exts : ['java', 'groovy'], exes : []},
                javascript : { exts : ['js'], exes : ['node']},
                json : {exts: ['json'], exes: []},
                lisp : { exts : ['lisp', 'scm', 'lsp'], exes : []},
                lua : { exts : ['lua'], exes : []},
                markdown : { exts: ['md'], exes : []},
                matlab : { exts : [], exes : []},
                mel : { exts : [], exes : []},
                nginx : { exts : [], exes : []},
                objectivec : { exts : ['m', 'mm'], exes : []},
                parser3 : { exts : [], exes : []},
                perl : { exts : ['pl'], exes : ['perl']},
                php : { exts : ['php', 'php3', 'phtml', 'phtm'], exes : []},
                profile : { exts : [], exes : []},
                python : { exts : ['py', 'pyw'], exes : ['python']},
                r : { exts : [], exes : []},
                rib : { exts : [], exes : []},
                rsl : { exts : [], exes : []},
                ruby : { exts : ['rb'], exes : ['ruby']},
                rust : { exts : [], exes : []},
                scala : { exts : ['scala'], exes : []},
                smalltalk : { exts : [], exes : []},
                sql : { exts : ['sql'], exes : []},
                tex : { exts : ['tex'], exes : []},
                vala : { exts : [], exes : []},
                vbscript : { exts : ['vb'], exes : []},
                vhdl : { exts : ['vhdl'], exes : []},
                xml : { exts : ['xml', 'html', 'htm', 'xhtml', 'shtml', 'shtm', 'jsp', 'jspf', 'tag', 'jspx', 'dtd', 'xsd', 'xsl', 'xslt'], exes : []}
            },
            knownExtensions = {
                'txt': 'text',
                'log': 'text'
            },
            knownExecutables = {};

        // convert languages => extensions
        _.forEach(knownLanguages, function (config, lang) {
            _.forEach(config.exts, function (ext) {
                knownExtensions[ext] = lang;
            });
            _.forEach(config.exes, function (exe) {
                knownExecutables[exe] = lang;
            });
        });

        function determineLanguageFromExtension(extension) {
            return knownExtensions[extension] || null;
        }

        function determineLanguageFromFirstLine(line) {
            var match = pattern.exec(line);
            if (match) {
                var exe = match[1];
                if (exe === 'env') {
                    exe = match[2];
                }
                return knownExecutables[exe] || null;
            } else {
                return null;
            }
        }

        return function (data, path) {
            var filename = path.getName(),
                extension = filename.lastIndexOf('.') === -1 ? '' : filename.substring(filename.lastIndexOf('.') + 1).toLowerCase();
            return determineLanguageFromExtension(extension) || (data.start === 0 && data.size > 1 ? determineLanguageFromFirstLine(data.lines[0].text) : null);
        };
    })();

    function lineToHtml(line) {
        var lineHtml = AJS.escapeHtml(line.text);
        return lineOpts.lineStart + (lineHtml || lineOpts.emptyLine) + lineOpts.lineEnd(line);
    }

    function lineArrayToHtml(lines) {
        return _.map(lines, lineToHtml).join('');
    }

    function lineNumbersToHtml(start, size) {
        var html = '';
        for (var i = start, end = start + size; i < end; i++) {
            html += '<pre id="l' + i + '"><a href="#' + i + '">' + i + '</a></pre>';
        }
        return html;
    }


    function SourceView(tableSelector, scrollPaneSelector) {
        PagedScrollable.call(this, scrollPaneSelector, {
            pageSize: pageSize,
            dataLoadedEvent: 'stash.feature.sourceview.dataLoaded'
        });

        this.$table = $(tableSelector);
        this.$linesContainer = this.$table.children('.line-numbers');
        this.$linesBlock = this.$linesContainer.children('.numbers');
        this.$sourceBlock = this.$table.find('.code');
        this.$source = $('<code />').appendTo(this.$sourceBlock);

        // Bottom padding on the table cells so the scrollbar doesn't cover anything
        this.scrollbarPadding = 0;

        this.$spinnerTarget = $('<div id="file-content-spinner" />');
        this.$table.after(this.$spinnerTarget);

        if (isIE9) {
            this.$table.addClass('padded-scrollbar');
            this.scrollbarPadding = parseInt(this.$linesBlock.css('padding-bottom'), 10);
        }
    }
    $.extend(SourceView.prototype, PagedScrollable.prototype);

    SourceView.prototype.init = function(fileChange, options) {
        var targetedItem = options && options.targetItem;

        this.reset();

        this.fileChange = fileChange;
        this.startingLine = targetedItem != null ? Math.min(targetedItem + 1, maxLinesDisplayed) : null;

        var self = this;
        this.onRequestedRevisionData = function() {
            self.reset();
            self.$table.removeClass('fully-loaded');
            self.$spinnerTarget.show().spin("large");
        };
        eve.on('stash.layout.branch.requestedRevisionData', this.onRequestedRevisionData);

        return PagedScrollable.prototype.init.call(this, {targetedItem: targetedItem, loadedRange: new LoadedRange(maxLinesDisplayed)});
    };

    // in Chrome:
    // quickEmpty 5000 lines - 70ms
    // $node.empty() - 350ms

    // WARNING: Only use if you can guarantee that no data or event listeners are stored against this node or any of its children.
    // Using this otherwise will cause a memory leak
    function quickEmpty(node) {
        while(node.hasChildNodes()) node.removeChild(node.firstChild);
    }

    SourceView.prototype.reset = function() {
        this.startingLine = 0;

        this.fileChange = undefined;
        this.language = undefined;

        quickEmpty(this.$linesBlock[0]);
        quickEmpty(this.$source[0]);

        this.fontSizeHandler && eve.unbind('stash.util.events.fontSizeChanged', this.fontSizeHandler);

        if(this.onRequestedRevisionData) {
            eve.unbind('stash.layout.branch.requestedRevisionData', this.onRequestedRevisionData);
        }

        this.$linesBlock.off('click.source-view', 'a');

        PagedScrollable.prototype.reset.call(this);
    };

    SourceView.prototype.destroy = function() {
        this.reset();
        this.$spinnerTarget.remove();
        if (this.$overCapacityMessage) {
            this.$overCapacityMessage.remove();
            this.$overCapacityMessage = null;
        }
    };

    //@Override
    SourceView.prototype.requestData = function (start, limit) {
        this.$spinnerTarget.spin('large');

        var self = this,
            data = {
                start: start,
                limit: limit
            };

        if (this.includeBlame) {
            data.blame = true;
        }

        var xhr = getData(this.fileChange.getPath(), this.fileChange.getCommitRange().getUntilRevision(), data),
            pipedXhr = xhr.pipe(function (data) {
                if (data.errors && data.errors.length) {
                    return $.Deferred().rejectWith(this, [this, null, null, data]);
                }

                return data;
            }).
            fail(function (xhr, text, error, data) {
                self.$spinnerTarget.spinStop();
            });

        return pipedXhr.promise(xhr);
    };

    SourceView.prototype.handleErrors = function(errors) {
        if (errors) {
            this.$source.prepend(_.reduce(errors, function(seed, error) {
                var messageText = error && error.message || stash_i18n('stash.web.source.error.unknown', 'Source could not be loaded');
                var message = widget.aui.message.warning({
                    contents : AJS.escapeHtml(messageText)
                });
                return seed + message;
            }, ''));
            eve('stash.feature.sourceview.onError', this, { errors : errors });
        }
    };

    SourceView.prototype.onBinary = function (data) {
        var $container = this.$table.find(".binary-container");
        binaryView.renderBinaryFile($("<div>"), data.path, this.fileChange.getCommitRange())
            .appendTo($container);
        this.$spinnerTarget.spinStop();
        eve('stash.feature.sourceview.onBinary', this, { path : data.path });
    };

    SourceView.prototype.onEmptyFile = function (data) {
        this.$table.addClass("empty-file").html(stash.feature.fileContent.emptyFile());
        this.$spinnerTarget.remove();
        eve('stash.feature.sourceview.onEmptyFile', this, data);
    };

    //@Override
    SourceView.prototype.onDataLoaded = function (start, limit, data) {

        var self = this;

        if (binaryView.shouldRenderBinary(data)) {
            this.$table
                .find(".source-container, .line-numbers").hide()
                .end()
                .find(".binary-container").show();
            this.onBinary(data);
        } else if (data.lines && data.lines.length === 0) {
            this.$table
                .find(".source-container, .line-numbers").show()
                .end()
                .find(".binary-container").hide().empty();
            this.onEmptyFile(data);
        } else {
            PagedScrollable.prototype.onDataLoaded.call(this, start, limit, data);
        }

        this.$spinnerTarget.spinStop();
        if (self.loadedRange.reachedEnd() || data.binary) {
            self.$table.addClass('fully-loaded');
            self.$spinnerTarget.hide();
            if (self.loadedRange.reachedCapacity() && !self.$overCapacityMessage) {
                self.$overCapacityMessage = $(widget.aui.message.warning({
                    contents : stash_i18n('stash.web.source.overcapacity',
                        'This file is too large to render. <a href="{0}">Download the full file</a>.',
                        navBuilder.currentRepo().browse()
                            .path(self.fileChange.getPath()).at(self.fileChange.getUntilRevision().getId()).raw().build()
                    )
                }));
                self.$table.after(self.$overCapacityMessage);
            }
        }
    };

    //@Override
    SourceView.prototype.attachNewContent = function (data, attachmentMethod) {

        var language = this.language = this.language || (isSyntaxHighlightingDisabled ?
                    NO_HIGHLIGHT_LANGUAGE :
                    getLanguage(data, this.fileChange.getPath()) || NO_HIGHLIGHT_LANGUAGE);

        var codeHtml;
        if (language === NO_HIGHLIGHT_LANGUAGE) {
            codeHtml = lineArrayToHtml(data.lines);
        } else {
            var code = _.map(data.lines,
                function (line) {
                    return line.text;
                }).join('\n') + '\n',
                highlightedCode = hljs.highlight(language, code).value;

            codeHtml = htmlUtil.mergeStreams(code, htmlUtil.lineNodeStream(data.lines, null, lineOpts), htmlUtil.highlightJsNodeStream(highlightedCode))
                .replace(/\n/g, '');
        }

        htmlUtil.quickNDirtyAttach(this.$source[0], codeHtml, attachmentMethod);

        if (attachmentMethod === 'prepend') {
            this._resizePlaceholder();
        }
        htmlUtil.quickNDirtyAttach(this.$linesBlock[0], lineNumbersToHtml(Number(data.start) + 1, Number(data.size)), attachmentMethod);
    };

    SourceView.prototype._resizePlaceholder = function() {
        var self = this,
            loadedStart = self.loadedRange.start,
            loadedEnd = self.loadedRange.nextPageStart;

        if (loadedStart > 0) {
            var lineBlock = self.$linesBlock[0],
                scrollbarPadding = self.scrollbarPadding,
                $placeholder;

            if (!($placeholder = self.$precedingSourcePlaceholder)) {
                $placeholder = self.$precedingSourcePlaceholder = $('<div/>').prependTo(self.$sourceBlock).add($('<div/>').prependTo(self.$linesContainer));
            }

            //set the placeholder height to 0 to avoid it affecting the lineBlock height when we grab that later.
            $placeholder.height(0);
            $placeholder.height(((lineBlock.offsetHeight - scrollbarPadding) / (loadedEnd - loadedStart)) * loadedStart);
            $placeholder.addClass('source-placeholder');

            // setTimeout required to wait for container to appear on page before it can be appended to
            setTimeout(function() {
                $('.source-container').find('.source-placeholder').append(self.$spinnerTarget);
                self.$spinnerTarget.spin('large');
            }, 0);

        } else if (self.$precedingSourcePlaceholder) {
            self.$precedingSourcePlaceholder.remove();
            self.$precedingSourcePlaceholder = null;
        }
    };

    //@Override
    SourceView.prototype.onFirstDataLoaded = function (start, limit, data) {

        var self = this;

        // Currently, we are adding all previous line numbers to the DOM when you deep link.
        // We do that when we render the deeplinked page
        if (start > 0) {
            this._resizePlaceholder();
        }

        eve.on('stash.util.events.fontSizeChanged', this.fontSizeHandler = function() { self._resizePlaceholder(); });

        if (this.startingLine) {
            this.selectLine(this.startingLine, true);
        }

        this.$linesBlock.on('click.source-view', 'a', function () {
            eve('stash.feature.sourceview.selectedLineChanged', self, this.parentNode.id.match(/\d+/)[0]);
        });

        if (isSyntaxHighlightingDisabled && data.lines.length) {
            var language = getLanguage(data, self.fileChange.getPath()) || NO_HIGHLIGHT_LANGUAGE;
            if (language !== NO_HIGHLIGHT_LANGUAGE) {
                var $syntaxHighlightWarning = $(stash.feature.fileContent.syntaxHighlightToolbarWarning()).appendTo('.source-commands .aui-toolbar2-primary');
                new AJS.InlineDialog($syntaxHighlightWarning.find('.dialog-trigger'), 'highlight-demo', function(contents, trigger, showPopup) {

                    var $container = $('<div class="syntax-highlight-warning-dialog"></div>');
                    $container.html(stash_i18n(
                        'stash.web.sourceview.warning.nosyntaxhighlight.detail',
                        'Syntax highlighting is not supported in Internet Explorer 8. For a better experience, consider {0}upgrading your browser{1}.',
                        '<a href="http://www.browseryoulovedtohate.com" target="_blank">',
                        '</a>'
                    ));

                    contents.html($container);
                    showPopup();
                }, {
                    hideDelay : 500,
                    width : 350
                });
            }
        }

        PagedScrollable.prototype.onFirstDataLoaded.call(this);
    };

    SourceView.prototype.selectLine = function(lineNumber, initialPageLoad) {
        var $lineLink = this.$lineLink;
        if ($lineLink) {
            $lineLink.removeClass('target');
        }

        if (lineNumber) {
            $lineLink = this.$lineLink = this.$linesBlock.find('#l' + lineNumber);
            if ($lineLink.length) {
                // put the line at the top third.
                this.$scrollElement.scrollTop($lineLink.offset().top - ($(window).height() / 4));
                $lineLink.addClass('target');

                if (initialPageLoad) {
                    $lineLink.addClass('initial-target');
                }
            }
        }
    };

    return SourceView;
});
