define('feature/pull-request/pull-request-participants-list', [
    'jquery',
    'underscore',
    'eve',
    'model/page-state'
], function(
    $,
    _,
    eve,
    pageState
) {

    var participants = pageState.getPullRequest().getParticipants();

    function ParticipantsList($list, $container) {
        this.$list = $list;
        this.$container = $container;

        _.bindAll(this, 'addedParticipantHandler', 'approvalHandler');

        participants.on('change:approved', this.approvalHandler);
        participants.on('add', this.addedParticipantHandler);
    }

    ParticipantsList.prototype.addedParticipantHandler = function(participant) {
        this.$list.append("<li>" + stash.widget.avatarList.participantAvatar({
            participant: participant.toJSON(),
            extraClasses: 'participant-item',
            withName: true
        }) + "</li>");

        var $count = this.$container.find('.count');
        if ($count.length) {
            var count = parseInt($count.text(), 10);

            if (count === 1) {
                // Pluralise label
                var $label = this.$container.find('.label');
                $label.text(stash_i18n('stash.web.pull-request.participants.label.plural', 'Participants'));
            }

            $count.text(++count);
        } else {
            $('.participants-trigger').prepend('<strong class="count">1</strong> ');
        }

        this.$container.removeClass('hidden');
    };

    ParticipantsList.prototype.approvalHandler = function(participant) {
        var $avatars = this.$list.find(".user-avatar[data-username='" + participant.getUser().getName() + "']");
        $avatars.toggleClass("badge-hidden", !participant.getApproved());
    };

    ParticipantsList.prototype.destroy = function() {
        participants.off('add', this.addedParticipantHandler);
        participants.off('change:approved', this.approvalHandler);
    };

    return ParticipantsList;
});
