define('layout/repository', [
    'jquery',
    'eve',
    'model/page-state',
    'feature/repository/sticky-branches',
    'widget/quick-copy-text',
    'exports'
], function(
    $,
    eve,
    pageState,
    stickyBranches,
    quickCopyText,
    exports
) {

    function initRepositoryPageState(repositoryJson) {
        pageState.setRepository(repositoryJson);
        pageState.setProject(repositoryJson.project);
    }

    var getCloneUrlContainer = function () {
        return $('.repository-url');
    };

    var getCloneUrlProtocolTrigger = function () {
        return $('.repository-protocol');
    };

    /**
     * Input elements can't be sized to fit their contents, so we have to use a bit of javascript to do it for us.
     * This method creates a fake element to calculate the size of one monospace character and then sets the width
     * of the input element to be input.length * width. There are a few rounding issues on some browsers but should
     * be good enough for the most part.
     *
     * Finally, this method also binds events to focus/mouseup to automatically select the input text for the
     * convenience of the user who is going to want to copy the value.
     */
    var setupCloneUrlInput = function () {
        var $container = getCloneUrlContainer(),
            $cloneInput = $container.children("input"),
            $cloneProtocolTrigger = getCloneUrlProtocolTrigger(),
            $cloneProtocolDropdown = $('#' + $cloneProtocolTrigger.attr("aria-owns")),
            $cloneProtocolDropdownItems = $cloneProtocolDropdown.find('li');

        if($cloneProtocolTrigger.is('button')) {
            updateCloneProtocolTrigger($cloneProtocolTrigger, $cloneProtocolTrigger.text());
            updateCloneUrl($cloneInput, $cloneProtocolTrigger.attr('data-clone-url'), $container);
        } else {
            updateCloneProtocolTrigger($cloneProtocolTrigger, $cloneProtocolDropdownItems.first().children('a').text());
            updateCloneUrl($cloneInput, $cloneProtocolDropdownItems.first().attr('data-clone-url'), $container);
        }
        resizeCloneUrlInput($cloneInput, $container);

        $cloneProtocolDropdownItems.on('click', function(event) {
            updateCloneProtocolTrigger($cloneProtocolTrigger, $(this).text());
            updateCloneUrl($cloneInput, $(this).attr('data-clone-url'), $container);
            resizeCloneUrlInput($cloneInput, $container);
            event.preventDefault();
        });
    };

    var updateCloneProtocolTrigger = function(trigger, newLabel) {
        var $cloneProtocolLabel = trigger.children('span').remove(); // pull the icon span element out and store it temporarily
        trigger.text(newLabel).append($cloneProtocolLabel); // replace the dropdown trigger text and add the icon span back in
    };

    var updateCloneUrl = function(element, url, container) {
        element.val(url);
        resizeCloneUrlInput(element, container);
    };

    var resizeCloneUrlInput = function(element, container) {
        var $fakey = $("<span>").text(element.val()).appendTo(container),
            monospaceWidth = $fakey.width();
        $fakey.remove();

        var maxWidth = 440,
            inputWidth = monospaceWidth + (element.outerWidth() - element.width()); // cloneInput is box-sizing border-box
        element.width(Math.min(maxWidth, inputWidth));
    };

    var setupCreatePullRequestButton = function () {
        eve.on('stash.layout.branch.revisionRefChanged', function(revisionReference) {
            var params = {
                "projectKey": pageState.getProject().key,
                "repoSlug": pageState.getRepository().slug
            };

            if (!revisionReference.isDefault() && revisionReference.isBranch()) {
                params.sourceBranch = revisionReference.getDisplayId();
            }

            var $link = $(stash.widget.pullRequest.createPullRequestButton(params));
            $(".aui-page-header-actions .create-pull-request").replaceWith($link);
        });
    };

    exports.onReady = function(repositoryJson) {
        initRepositoryPageState(repositoryJson);
        setupCloneUrlInput();
        setupCreatePullRequestButton();
        quickCopyText.onReady();
        stickyBranches.onReady();
    };
});
