define('model/conflict',
    ['jquery', 'widget/path', 'model/modification-states'],
    function ($, Path, ModificationStates) {

        function Conflict(ourChangeOrJSON, theirChange) {
            if ($.isPlainObject(ourChangeOrJSON)) {
                return Conflict.fromJSON(ourChangeOrJSON);
            }

            this._ourChange = ourChangeOrJSON;
            this._theirChange = theirChange;
        }

        Conflict.fromJSON = function(jsonConflict) {
            var FileChange = require('widget/file-change'); //avoid circular dependency with FileChange
            return new Conflict(
                jsonConflict.ourChange && new FileChange(undefined, new Path(jsonConflict.ourChange.srcPath), new Path(jsonConflict.ourChange.path), jsonConflict.ourChange.type),
                jsonConflict.theirChange && new FileChange(undefined, new Path(jsonConflict.theirChange.srcPath), new Path(jsonConflict.theirChange.path), jsonConflict.theirChange.type)
            );
        };

        Conflict.prototype.getOurChange = function() {
            return this._ourChange;
        };

        Conflict.prototype.getTheirChange = function() {
            return this._theirChange;
        };

        Conflict.prototype.toJSON = function() {
            return {
                ourChange: this._ourChange.toJSON(),
                theirChange: this._theirChange.toJSON()
            };
        };

        Conflict.prototype.getConflictMessage = function() {
            //'our' describes the change that was made on the destination branch relative to a shared
            //ancestor with the incoming branch.
            //'their' describes the change that was made on the incoming branch relative to a shared
            //ancestor with the destination branch
            var destinationModState = this._ourChange && this._ourChange.getModificationState(),
                incomingModState = this._theirChange && this._theirChange.getModificationState();

            return (messageMatrix[incomingModState] && messageMatrix[incomingModState][destinationModState]) || '';
        };
        
        var messageMatrix = {};

        messageMatrix[ModificationStates.ADD] = {};
        messageMatrix[ModificationStates.ADD][ModificationStates.ADD]         = stash_i18n('stash.web.pull-request.diff.conflict.title.add.add',        'Conflict: Added file to Source, Added same file to Target');
        messageMatrix[ModificationStates.ADD][ModificationStates.RENAME]      = stash_i18n('stash.web.pull-request.diff.conflict.title.add.add',        'Conflict: Added file to Source, Renamed to same on Target');
        messageMatrix[ModificationStates.ADD][ModificationStates.MOVE]        = stash_i18n('stash.web.pull-request.diff.conflict.title.add.add',        'Conflict: Added file to Source, Moved to same on Target');

        messageMatrix[ModificationStates.MODIFY] = {};
        messageMatrix[ModificationStates.MODIFY][ModificationStates.MODIFY]   = stash_i18n('stash.web.pull-request.diff.conflict.title.modify.modify',  'Conflict: Modified on Source, Modified on Target');
        messageMatrix[ModificationStates.MODIFY][ModificationStates.RENAME]   = stash_i18n('stash.web.pull-request.diff.conflict.title.modify.rename',  'Conflict: Modified on Source, Renamed on Target');
        messageMatrix[ModificationStates.MODIFY][ModificationStates.MOVE]     = stash_i18n('stash.web.pull-request.diff.conflict.title.modify.move',    'Conflict: Modified on Source, Moved on Target');
        messageMatrix[ModificationStates.MODIFY][ModificationStates.DELETE]   = stash_i18n('stash.web.pull-request.diff.conflict.title.modify.delete',  'Conflict: Modified on Source, Deleted on Target');

        messageMatrix[ModificationStates.RENAME] = {};
        messageMatrix[ModificationStates.RENAME][ModificationStates.ADD]      = stash_i18n('stash.web.pull-request.diff.conflict.title.rename.add',     'Conflict: Renamed on Source, Added same to Target');
        messageMatrix[ModificationStates.RENAME][ModificationStates.RENAME]   = stash_i18n('stash.web.pull-request.diff.conflict.title.rename.rename',  'Conflict: Renamed on Source, Renamed on Target');
        messageMatrix[ModificationStates.RENAME][ModificationStates.MOVE]     = stash_i18n('stash.web.pull-request.diff.conflict.title.rename.move',    'Conflict: Renamed on Source, Moved on Target');
        messageMatrix[ModificationStates.RENAME][ModificationStates.DELETE]   = stash_i18n('stash.web.pull-request.diff.conflict.title.rename.delete',  'Conflict: Renamed on Source, Deleted on Target');

        messageMatrix[ModificationStates.MOVE] = {};
        messageMatrix[ModificationStates.MOVE][ModificationStates.ADD]        = stash_i18n('stash.web.pull-request.diff.conflict.title.move.add',       'Conflict: Moved on Source, Added same to Target');
        messageMatrix[ModificationStates.MOVE][ModificationStates.RENAME]     = stash_i18n('stash.web.pull-request.diff.conflict.title.move.rename',    'Conflict: Moved on Source, Renamed on Target');
        messageMatrix[ModificationStates.MOVE][ModificationStates.MOVE]       = stash_i18n('stash.web.pull-request.diff.conflict.title.move.move',      'Conflict: Moved on Source, Moved on Target');
        messageMatrix[ModificationStates.MOVE][ModificationStates.DELETE]     = stash_i18n('stash.web.pull-request.diff.conflict.title.move.delete',    'Conflict: Moved on Source, Deleted on Target');

        messageMatrix[ModificationStates.DELETE] = {};
        messageMatrix[ModificationStates.DELETE][ModificationStates.MODIFY]   = stash_i18n('stash.web.pull-request.diff.conflict.title.delete.modify',  'Conflict: Deleted on Source, Modified on Target');
        messageMatrix[ModificationStates.DELETE][ModificationStates.RENAME]   = stash_i18n('stash.web.pull-request.diff.conflict.title.delete.rename',  'Conflict: Deleted on Source, Renamed on Target');
        messageMatrix[ModificationStates.DELETE][ModificationStates.MOVE]     = stash_i18n('stash.web.pull-request.diff.conflict.title.delete.move',    'Conflict: Deleted on Source, Moved on Target');

        return Conflict;
    });
