define('model/revision-reference', [
    'backbone-brace'
], function(
    Brace
) {

    var Type = {
        TAG: {
            id: "tag",
            name: stash_i18n('stash.web.revisionref.tag.name', 'Tag')
        },
        BRANCH: {
            id: "branch",
            name: stash_i18n('stash.web.revisionref.branch.name', 'Branch')
        },
        COMMIT: {
            id: "commit",
            name: stash_i18n('stash.web.revisionref.commit.name', 'Commit')
        },
        isTag: function (o) {
            return o && (o === Type.TAG.id || o.id === Type.TAG.id);
        },
        isBranch: function (o) {
            return o && (o === Type.BRANCH.id || o.id === Type.BRANCH.id);
        },
        isCommit: function(o) {
            return o && (o === Type.COMMIT.id || o.id === Type.COMMIT.id);
        },
        from: function (o) {
            if (Type.isTag(o)) {
                return Type.TAG;
            } else if (Type.isBranch(o)) {
                return Type.BRANCH;
            } else if (Type.isCommit(o)) {
                return Type.COMMIT;
            }
            window.console && console.error("Unknown RevisionReference type " + o);
            return null;
        }
    };

    /**
     * RevisionReference is a "reference" to a commit. This could be a branch, or a tag, or a direct
     * commit hash. It should be similar to the server-side "com.atlassian.stash.repository.Ref" class.
     * 
     */
    var RevisionReference = Brace.Model.extend({
        namedAttributes : {
            'id' : null,
            'displayId' : null,
            'type' : Type.from,
            'isDefault' : null,
            'latestChangeset' : null,
            'repository' : null
        },
        initialize : function() {
            if (!this.getDisplayId()) {
                this.setDisplayId(this.getId());
            }
            if (!this.getIsDefault()) {
                this.setIsDefault(false);
            }
        },
        getTypeName : function() {
            return this.getType().name;
        },
        isDefault : function() {
            return this.getIsDefault() || false;
        },
        isTag : function() {
            return Type.isTag(this.getType());
        },
        isBranch : function() {
            return Type.isBranch(this.getType());
        },
        isCommit : function() {
            return Type.isCommit(this.getType());
        }
    }, {
        fromChangeset : function(changeset) {
            return new RevisionReference({
                id : changeset.id,
                displayId : changeset.displayId,
                type : Type.COMMIT,
                isDefault : false
            });
        }
    });

    RevisionReference.type = Type;

    return RevisionReference;
});
