define('page/commits',
    ['jquery', 'aui', 'eve', 'memoir', 'util/navbuilder', 'util/ajax', 'layout/branch', 'model/revision-reference', 'feature/commits/commits-table', 'exports'],
    function($, AJS, eve, memoir, navBuilder, ajax, branch,  RevisionReference, CommitsTable, exports) {
        var atRevisionRef,
            commitsTable;

        function getCommitsUrlBuilder(atRevRef) {
            atRevRef = atRevRef || atRevisionRef;
            var builder = navBuilder.currentRepo().commits();
            if (!atRevRef.isDefault()) {
                builder = builder.withParams({ until : atRevRef.getId() });
            }
            return builder;
        }

        function bindKeyboardShortcuts() {
            commitsTable.initShortcuts();

            eve.on('stash.widget.keyboard-shortcuts.register-contexts', function(keyboardShortcuts) {
                keyboardShortcuts.enableContext('commits');
            });

            eve.on('stash.keyboard.shortcuts.requestMoveToNextHandler',     commitsTable.bindMoveToNextHandler);
            eve.on('stash.keyboard.shortcuts.requestMoveToPreviousHandler', commitsTable.bindMoveToPreviousHandler);
            eve.on('stash.keyboard.shortcuts.requestOpenItemHandler',       commitsTable.bindOpenItemHandler);
            eve.on('stash.keyboard.shortcuts.requestToggleMergesHandler',   commitsTable.bindToggleMergesHandler);

            eve.on('stash.widget.branchselector.dialogShown', function () {
                eve('stash.keyboard.shortcuts.disableOpenItemHandler');
            });
            eve.on('stash.widget.branchselector.dialogHidden', function () {
                eve('stash.keyboard.shortcuts.enableOpenItemHandler');
            });
        }

        exports.onReady = function(atRevisionRefJSON) {
            atRevisionRef = new RevisionReference(atRevisionRefJSON);
            commitsTable = new CommitsTable(getCommitsUrlBuilder);
            commitsTable.init();

            eve.on('stash.widget.branchselector.revisionRefChanged', function(newAtRevisionRef) {
                if (atRevisionRef !== newAtRevisionRef) {
                    memoir.pushState(newAtRevisionRef.toJSON(), null, getCommitsUrlBuilder(newAtRevisionRef).build());
                }
            });

            eve.on('memoir.changestate', function(e) {
                var state = e.state;
                if (state) {
                    atRevisionRef = new RevisionReference(state);
                    commitsTable.update();
                    eve("stash.page.commits.revisionRefChanged", null, atRevisionRef);
                }
            });

            bindKeyboardShortcuts();

            memoir.initialState(atRevisionRef.toJSON());
        };
    });
