define('page/project/permissions', ['jquery', 'exports', 'util/ajax', 'util/navbuilder', 'widget/submit-spinner', 'widget/permission-table'],
function($, exports, ajax, nav, SubmitSpinner, permissionTable) {
    exports.onReady = function(permissions) {
        /* utility functions */
        function permsNavBuilder() {
            return nav.rest().project($("#projectKey").val()).permissions();
        }

        function updatePermission(permsNavBuilder, allow) {
            return ajax.rest({
                type: 'POST',
                url: permsNavBuilder.allow(allow).build()
            });
        }

        function getPermissions() {
            var selectedValue = getLicensedUserPermsRadios().filter(":checked").val();

            var perms = [];
            if (selectedValue === 'readwrite') {
                perms.push('read');
                perms.push('write');
            } else if (selectedValue === 'read') {
                perms.push('read');
            }
            return perms;
        }

        function getLicensedUserPermsRadios() {
            return $("input:radio[name=licensed-users-permissions]");
        }

        function updateRegisteredUsers(spinner) {
            spinner.show();

            var perms = getPermissions();
            updatePermission(permsNavBuilder().projectRead().all(), $.inArray('read', perms) > -1).pipe(function() {
                return updatePermission(permsNavBuilder().projectWrite().all(), $.inArray('write', perms) > -1).fail(function() {
                    // read succeeded, but write failed
                    getLicensedUserPermsRadios().filter("[value='read']").attr("checked", true);
                });
            }).always(function() {
                setTimeout(function() {
                    spinner.remove();
                }, 250);
            });
        }

        /* event handlers */
        getLicensedUserPermsRadios().click(function() {
            updateRegisteredUsers(new SubmitSpinner($(this).next('span')));
        });

        /* permission table*/
        var permControls = permissionTable.initialise(
            nav.project($("#projectKey").val()).permissions(),
            permissions,
            'PROJECT_ADMIN' //If the user can see this page, then the UI should act like they have project admin permissions
        );
    };
});
