define('page/pull-request/view/pull-request-view-overview', [
    'jquery',
    'util/navbuilder',
    'model/page-state',
    'lib/jsuri',
    'feature/comments/comment-tips',
    'feature/pull-request/pull-request-activity',
    'feature/pull-request/pull-request-participants-list',
    'widget/watch'
], function (
    $,
    nav,
    pageState,
    Uri,
    commentTips,
    PullRequestActivity,
    ParticipantsList,
    Watch
) {
    var currentEl;
    var activity;
    var canMerge = true;
    var watchButton;
    var participantsList;

    function showCantMergeBanner(el) {
        var mergeConflictBanner = $(stash.widget.pullRequest.mergeConflictBanner({
            extraClasses: 'transparent'
        }))
        .prependTo(el)
        .find('.manual-merge')
        .click(function(e) {
            e.preventDefault();
            eve('stash.pull-request.show.cant.merge.help');
        })
        .end();

        setTimeout(function(){ mergeConflictBanner.removeClass('transparent'); }, 0); //Let the message get rendered before starting the fade in.
    }

    eve.on('stash.pull-request.cant.merge', function(pullRequest, conflicted) {
        if (canMerge) {
            canMerge = false;
            if (conflicted && currentEl) {
                showCantMergeBanner(currentEl);
            }
        }
    });

    return {
        load : function(el) {
            currentEl = el;

            el.innerHTML = stash.page.pullRequest.viewOverview({
                pullRequest : pageState.getPullRequest(),
                author: pageState.getPullRequest().getAuthor().getUser().toJSON(),
                createdDate: pageState.getPullRequest().getCreatedDate(),
                description: pageState.getPullRequest().getDescription(),
                descriptionAsHtml: pageState.getPullRequest().getDescriptionAsHtml(),
                currentUser: pageState.getCurrentUser(),
                commentTips: commentTips.tips
            });

            // Ensures the general comment at the top of the page is wrapped into a expanding area.
            // Note that the other general comments and the diff comments are not in the page at this stage;
            // they will be fetched asynchronously below (through a request triggered in activity.init() below).
            $(el).find('textarea.expanding').expandingTextarea();

            if (!canMerge) {
                showCantMergeBanner(el);
            }

            watchButton = new Watch($('.watch a'), nav.rest().currentPullRequest().watch().build());
            participantsList = new ParticipantsList($('#participants-dropdown ul'), $('.participants.plugin-item'));

            var uri = new Uri(window.location);
            var fromType = uri.getQueryParamValue('commentId') ? 'comment' : 'activity';
            var fromId = uri.getQueryParamValue('commentId') || uri.getQueryParamValue('activityId');

            activity = new PullRequestActivity($(el).find('.pull-request-activity'), pageState.getPullRequest(), fromType, fromId, {
                scrollableElement:window
            });

            activity.init();
        },
        unload : function(el) {
            activity.reset();
            activity = null;
            $(el).empty();
            currentEl = null;
            watchButton.destroy();
            watchButton = null;
            participantsList.destroy();
            participantsList = null;
        },
        keyboardShortcutContexts : ['pull-request-overview']
    };
});
