define('page/pull-request/pull-request-view', [
    'aui',
    'jquery',
    'eve',
    'memoir',
    'util/feature-loader',
    'util/navbuilder',
    'model/page-state',
    'exports'
], function (
    AJS,
    $,
    eve,
    memoir,
    FeatureLoader,
    navBuilder,
    pageState,
    exports
) {
    /**
     * Handles the case where a user has clicked "unwatch" from an email. We show a message that
     * they have been marked as no longer watching this pull request.
     */
    function showUnwatchedMessage() {
        $(window).on('load', function() {
            var dialog = new AJS.Dialog({
                width: 450,
                height: 200,
                id: 'unwatch-msg-dialog',
                closeOnOutsideClick: false,
                keypressListener: function (e) {
                    e.stopImmediatePropagation();
                    if (e.keyCode === $.ui.keyCode.ENTER || e.keyCode === $.ui.keyCode.ESCAPE) {
                        dialog.remove();
                    }
                }
            });

            dialog.addHeader(stash_i18n('stash.web.pullrequests.unwatched.header', 'Stopped watching'), 'unwatch-dialog-header');

            var $content = $('<div/>').text(
                stash_i18n('stash.web.pullrequest.unwatched.content',
                    'Notifications for this pull request will no longer be sent to you.'));
            dialog.addPanel('$content', $content, 'unwatch-dialog-content');
            dialog.addLink(stash_i18n('stash.web.button.done', 'Done'), function (dialog) {
                dialog.remove();
            }, 'unwatch-msg-dialog-done-link');

            dialog.show();
        });
    }

    var loader = new FeatureLoader({
        loadedEvent : 'stash.page.pull-request.view.contextLoaded',
        unloadedEvent : 'stash.page.pull-request.view.contextUnloaded',
        requestedEvent : 'stash.page.pull-request.view.contextRequested'
    });

    exports.registerHandler = $.proxy(loader.registerHandler, loader);
    exports.onReady = function(
        startingSection,
        contentSelector,
        unwatched,
        diffTreeHeaderWebItems,
        commitsTableWebSections,
        maxChanges
    ) {
        pageState.extend("pullRequestViewInternal", function() {
            return {
                diffTreeHeaderWebItems : diffTreeHeaderWebItems,
                commitsTableWebSections : commitsTableWebSections,
                maxChanges : maxChanges
            };
        });

        // TODO: Consider Jason's idea of contexts. Lots of weirdness to flesh out with 
        // TODO: the best API for this stuff.

        loader.registerHandler(
            'stash.pull-request.nav.diff',
            /^[^\?\#]*pull-requests\/\d+\/diff/,
            'page/pull-request/view/pull-request-view-diff');
        loader.registerHandler(
            'stash.pull-request.nav.overview',
            /^[^\?\#]*pull-requests\/\d+\/overview/,
            'page/pull-request/view/pull-request-view-overview');
        loader.registerHandler(
            'stash.pull-request.nav.commits',
            /^[^\?\#]*pull-requests\/\d+\/commits/,
            'page/pull-request/view/pull-request-view-commits');

        eve.on('stash.widget.keyboard-shortcuts.register-contexts', function(keyboardShortcuts) {
            loader.setKeyboardShortcuts(keyboardShortcuts);
            keyboardShortcuts.enableContext('pull-request');
        });

        if (unwatched) {
            showUnwatchedMessage();
        }

        eve.on('stash.layout.pull-request.urlRequested', function(url) {
            if (url !== window.location.href) {
                memoir.pushState(null, '', url);
            }
        });

        loader.init($(contentSelector).get(0));
    };
});
