define('page/repositoryGeneralSettings',
['jquery', 'aui', 'util/ajax', 'util/navbuilder', 'widget/confirm-dialog', 'model/page-state', 'widget/branch-selector-dialog', 'feature/repository/cloneUrlGen', 'exports'],
function($, AJS, ajax, nav, ConfirmDialog, pageState, BranchSelectorDialog, cloneUrlGen, exports) {

    var branchSelectorDialog;

    function initDeleteButton(deleteButtonSelector) {
        var repo = pageState.getRepository();

        var $panelContent =
            $("<p>").html(stash_i18n("stash.web.repository.delete.confirm", "Are you sure that you want to delete {0}?", "<b>" + AJS.escapeHtml(repo.slug) + "</b>"));
        var $panelContentDetail =
            $("<p>").html(stash_i18n("stash.web.repository.delete.confirm.detail", "It cannot be undone and all your code will be lost if it is not stored elsewhere."));

        $.merge($panelContent, $panelContentDetail);

        var confirmDialog = new ConfirmDialog({
            id : "delete-repository-dialog",
            titleText: stash_i18n('stash.web.repository.delete.title', 'Delete repository'),
            titleClass : 'warning-header',
            panelContent : $panelContent,
            submitText : stash_i18n('stash.web.button.delete', 'Delete')
        }, { type: 'DELETE' });

        confirmDialog.attachTo(deleteButtonSelector);

        confirmDialog.addConfirmListener(function(promise) {
           promise.pipe(function(data, status, xhr) {
               return ajax.poll({
                   url : $(deleteButtonSelector).attr('href'),
                   statusCode : {
                       '404' : function() {

                           window.location = nav.currentProject().withParams({
                               'deletedRepositoryName': repo.slug
                           }).build();

                           return false; // don't handle this globally.
                       }
                   }
               });
           });
        });
    }

    function initBranchSelector() {
        var $field = $("#defaultBranch");
        var $branchSelectorButton = $('.branch-selector-trigger');

        branchSelectorDialog = new BranchSelectorDialog($branchSelectorButton, "defaultBranchSelector", { branchesOnly : true, disableLinks: true });
        eve.on('stash.widget.branchselector.revisionRefChanged', function(revisionRef) {
            if (revisionRef) {
                $field.val(revisionRef.getId());
                branchSelectorDialog.updateSelectedRevisionReference(revisionRef);
            }
        });
    }

    exports.onReady = function(deleteButtonSelector) {
        initDeleteButton(deleteButtonSelector);
        initBranchSelector();
        cloneUrlGen.bindUrlGeneration("#name", "#clone-url > span");
    };
});
