define('widget/branch-selector-dialog', [
    'aui',
    'jquery',
    'widget/branch-selector',
    'model/page-state',
    'eve'
], function(
    AJS,
    $,
    BranchSelector,
    pageState,
    eve) {

    function BranchSelectorDialog(branchSelectorTriggerSelector, dialogId, branchSelectorOpts) {
        branchSelectorOpts = branchSelectorOpts || {};
        var branchy = this.branchSelector = new BranchSelector(pageState.getProject().key, pageState.getRepository().slug, branchSelectorOpts);

        var $button = this.$button = $(branchSelectorTriggerSelector);

        var inlineDialog = this.inlineDialog = AJS.InlineDialog($button, dialogId, function (content, trigger, showPopup) {
            content.html(stash.widget.branchSelector({ branchesOnly : branchSelectorOpts.branchesOnly }));
            content.addClass('branch-selector-contents');
            showPopup();
            // No way of determining that the popup has _actually_ been rendered yet.
            setTimeout(function () {
                branchy.init(content);
                branchy.populateBranchList();
                branchy.focusFilter();
            }, 0);
        }, {
            hideDelay:null,
            width:270,
            hideCallback: function() {
                if ($(document.activeElement).closest(branchy._context).length) {
                    // if the focus is inside the dialog, you get stuck when it closes.
                    document.activeElement.blur();
                }
                $(document).add(inlineDialog).off('keydown', hideOnEscape);
                eve('stash.widget.branchselector.dialogHidden', this);
            },
            initCallback: function() {
                // Bind to both the document and the inline dialog because we need to stop
                // the bubbling of the event before it reaches the document as to trigger
                // other ESC handlers. We listen for keydown because that is what AUI is doing
                // in dialog.js
                $(document).add(inlineDialog).on('keydown', hideOnEscape);
                eve('stash.widget.branchselector.dialogShown', this);
            },
            calculatePositions: function (popup, targetPosition, mousePosition, opts) {
                // CONSTANTS
                var ARROW_WIDTH = 16,
                    ARROW_HEIGHT = 8;

                var branchSelectorButton = targetPosition.target.closest('.branch-selector-trigger'),
                    buttonOffset = branchSelectorButton.offset(),
                    buttonWidth = branchSelectorButton.outerWidth(),
                    buttonHeight = branchSelectorButton.outerHeight();

                var popupLeft = buttonOffset.left,
                    arrowTop = -ARROW_HEIGHT + 1,
                    popupTop = buttonOffset.top + buttonHeight + ARROW_HEIGHT,
                    arrowLeft = (buttonWidth / 2) - (ARROW_WIDTH / 2);

                return {
                    displayAbove: false,
                    popupCss: {
                        left: popupLeft,
                        top: popupTop
                    },
                    arrowCss: {
                        top: arrowTop,
                        left: arrowLeft
                    }
                };
            }
        });

        //at a later date we might want to consider extending inline dialog wiht this behaviour
        function hideOnEscape(e) {
            if(e.keyCode === $.ui.keyCode.ESCAPE) {
                // If the branch selector is in a modal dialog, We want to only close the selector
                // They should be able to hit ESC again to close the modal dialog
                e.stopImmediatePropagation();
                inlineDialog.hide();
            }
        }

        //ALSO urgh, but inline dialogs api is kind of crappy
        eve.on("stash.widget.branchselector.revisionRefChanged", function (revisionReference) {
            if (this === branchy) {
                inlineDialog.hide();
                $button.focus();
            }
        });
    }

    BranchSelectorDialog.prototype.updateSelectedRevisionReference = function(revisionReference) {
        this.$button.attr("title", revisionReference.getTypeName() + ': ' + revisionReference.getDisplayId());
        this.$button.find(".ref-type-icon")
            .toggleClass('icon-branch', revisionReference.isBranch())
            .toggleClass('icon-tag', revisionReference.isTag());
        this.$button.find(".ref-name").text(revisionReference.getDisplayId());
    };

    return BranchSelectorDialog;
});
