define('widget/watch', [
    'jquery',
    'underscore',
    'eve',
    'util/ajax',
    'model/page-state'
], function(
    $,
    _,
    eve,
    ajax,
    pageState
) {

    function Watch($watchButton, url) {
        var self = this;
        this.url = url;
        this.$watch = $watchButton;
        this.isWatching = pageState.getIsWatching();

        this.$watch.on('click', triggerClicked);

        _.bindAll(this, 'toggleWatch', 'toggleUnwatch', 'toggleTrigger');

        eve.on('stash.widget.watch-button.adding', this.toggleWatch);
        eve.on('stash.widget.watch-button.removing', this.toggleUnwatch);
        eve.on('stash.widget.watch-button.add.fail', this.toggleUnwatch);
        eve.on('stash.widget.watch-button.remove.fail', this.toggleWatch);


        function triggerClicked(e) {
            e.preventDefault();

            function fireEvent(eventName) {
                eve(eventName, self, {
                    user: pageState.getCurrentUser()
                });
            }

            fireEvent(self.isWatching ? 'stash.widget.watch-button.removing' : 'stash.widget.watch-button.adding');

            return ajax.rest({
                url: self.url,
                type: self.isWatching ? 'DELETE' : 'POST',
                statusCode: {
                    '401' : function(xhr, textStatus, errorThrown, errors, dominantError) {
                        return $.extend({}, dominantError, {
                            title: stash_i18n("stash.web.pull-request.watch.error.401.title", "Could not watch pull request."),
                            message: stash_i18n("stash.web.pull-request.watch.error.401.message", "You do not have permission to watch this pull request."),
                            fallbackUrl: false,
                            shouldReload: true
                        });
                    },
                    '409' : function(xhr, textStatus, errorThrown, errors, dominantError) {
                        return $.extend({}, dominantError, {
                            title: stash_i18n("stash.web.pull-request.watch.error.409.title", "Could not watch pull request."),
                            fallbackUrl: false,
                            shouldReload: true
                        });
                    }
                }
            }).done(function() {
                self.isWatching = !self.isWatching;
                fireEvent(self.isWatching ? 'stash.widget.watch-button.added' : 'stash.widget.watch-button.removed');
            }).fail(function() {
                fireEvent(self.isWatching ? 'stash.widget.watch-button.remove.fail' : 'stash.widget.watch-button.add.fail');
            });
        }
    }

    Watch.prototype.toggleWatch = function() {
        this.toggleTrigger(true);
    };

    Watch.prototype.toggleUnwatch = function() {
        this.toggleTrigger(false);
    };

    // isWatching is the new watching state
    Watch.prototype.toggleTrigger = function (isWatching) {
        pageState.setIsWatching(isWatching);

        var triggerHtml = stash.feature.pullRequest.overview.plugins.watchLabel({isWatching : isWatching});

        this.$watch.fadeOut(200, function() {
            $(this).html(triggerHtml).fadeIn(200);
        });
    };

    Watch.prototype.destroy = function() {
        eve.off('stash.widget.watch-button.adding', this.toggleWatch);
        eve.off('stash.widget.watch-button.removing', this.toggleUnwatch);
        eve.off('stash.widget.watch-button.add.fail', this.toggleUnwatch);
        eve.off('stash.widget.watch-button.remove.fail', this.toggleWatch);
        this.url = null;
        this.$watch = null;
        this.isWatching = null;
    };

    return Watch;
});
